/*
    Copyright 2009 Sebastian Kłęk, Piotr Domagala and Jakub Mielczarek
    This file is part of the Filtertool.

    Filtertool is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*! \author Sebastian Kłęk
  * \author Piotr Domagala
  * \author Jakub Mielczarek
  */

#include <boost/program_options.hpp>
#include "DataOperations.h"

using namespace std;
namespace po = boost::program_options;
int main(int argc, char* argv[]) {
	bool created = false;
	DataHandler *a;
	string command = "";
	int order[6] = {0, 0, 0, 0, 0, 0};
	int j = 0;

	for(int i = 0; i < argc; i++){
		command = command + argv[i] + " ";
		if (string(argv[i]).compare("-c") == 0) {
			order[j] = 1;
			j++;
		} else if (string(argv[i]).compare("-f") == 0) {
			order[j] = 2;
			j++;
		} else if (string(argv[i]).compare("-g") == 0) {
			order[j] = 3;
			j++;
		} else if (string(argv[i]).compare("-p") == 0) {
			order[j] = 4;
			j++;
		} else if (string(argv[i]).compare("-s") == 0) {
			order[j] = 5;
			j++;
		}
	}
	try {
		po::options_description argless("Options without arguments");
		argless.add_options()
			( "help,h", "Standard help" )
			( "tab", "Separate values with tabulators instead of spaces" )
			( "version,v", "Display version" )
			( "float", "Use float as data type" )
			( "int", "Use int as data type" )
			( "join,j", "Join input" )
			( "merge,m", "Merge all data" )
			( "col", "Calculation in columns instead in lines" );
		po::options_description withargs( "Options with arguments" );
		string files_in;
		string file_out;
		withargs.add_options()
			( "input-files,i", po::value< string>(&files_in), "Give the names of files separated by \',\'\n" )
			( "output-files,o", po::value< string >(&file_out), "Specifies otupt files names separated by \',\'. Without this option all will be printed to standard output\n" )
			( "filter,f", po::value< string >() , "Use this option to select values.\n For example \'-f 1,gt2,0,1,2\'" )
			( "tolerance,t", po::value< double >(), "Tolerance of comparisons used in filter for example: \'-t 1e-10\'\n" )
			( "calculate,c", po::value< string >() , "Argument: s - sum, a - average, m - median, \n d - standard deviation (sample standard deviation)\n c - count For example: \'-c a,s,m,c\'\n" )
			( "group,g", po::value< string >() , "Use this option to group lines by values in specific column or by order in files. \n For example: \'-g 2\', \'-g f\'\n" )
			( "print,p", po::value< string >(), "Print only selected columns/parameters for example:\n \'-p 1,2,E\' where E is name of parameter\n" )
			( "swap,s", po::value< string >(), "Swaps two columns in selected file: - s 1,2,3 means swap first and second column in third file\n" )
			( "comments" , po::value< string >(), "Argument: off/all/1\n")
			( "prec" , po::value< int >(), "Sets printing precision of all values\n e.g. \'--prec 10\'\n")
			( "blanks,b" , po::value< string >(), "Set string displayed in place of missing values");
		po::options_description helpInfo( "Available options" );
		helpInfo.add( argless ).add( withargs );
		po::options_description parseCmdLine;
		parseCmdLine.add( argless ).add( withargs );
		po::variables_map vm;
		try {
			store( po::command_line_parser(argc,argv).options(parseCmdLine).run(), vm );
		}
		catch( exception& e_ ) {
			cerr << "Error while parsing command-line: " << e_.what() << "\n";
			return -1;
		}
		po::notify( vm );
		if( vm.count("int") ) {
			clog << "Using Integers as data type" << endl;
			a = new DataOperations<int>();
			created = true;
		}
		else if( vm.count("float") ) {
			clog << "Using Float as data type" << endl;
			a = new DataOperations<float>();
			created = true;
		}
		if (!created) {
			a = new DataOperations<double>();
		}
		if( vm.count("help") || argc == 1 ) {
			cout << "\nusage: ./program [OPTION <arg1,arg2,...>] ...\n\n";
			cout << helpInfo << "\n";
			cout << "For more info type 'man filtertool'.\n";
			return 0;
		}
		if( vm.count("version") ) {
			cout << "version 1.1\n";
			return 0;
		}
		if( vm.count("number") ) {
			cout << "version 1.1\n";
			return 0;
		}
		if ( vm.count("tab") ) {
			a->tab();
		}if ( vm.count("blanks") ) {
			a->blanks( vm["blanks"].as<string>() );
		}
		if( vm.count("join") ) {
			a->join();
		}
		if( vm.count("col") ) {
			a->col();
		}
		if( vm.count("comments") ) {
			a->comments( vm["comments"].as<string>() );
		}
		if( vm.count("input-files") ) {
			a->parseFiles(files_in);
		}
		else if( vm.count("input-files") == 0 ) {
			cerr << "read from stdin" << endl;
			a->load();
		}
		if( vm.count("output-files") ) {
			a->setOutputFiles( vm["output-files"].as<string>() );
		}
		if( vm.count("tolerance") ) {
			a->tol( vm["tolerance"].as<double>() );
		}
		if( vm.count("prec") ) {
			a->prec( vm["prec"].as<int>() );
		}
		if( vm.count("merge") ) {
			a->merge();
		}
		j = 0;
		while (order[j]){
			switch (order[j]){
				case 1:
					a->calculate( vm["calculate"].as<string>() );
					break;
				case 2:
					a->setFilter( vm["filter"].as<string>() );
					break;
				case 3:
					a->group( vm["group"].as<string>() );
					break;
				case 4:
					a->print( vm["print"].as<string>() );
					break;
				case 5:
					a->swap( vm["swap"].as<string>() );
					break;
				default:
					break;
			}
			j++;
		}
	}
	catch( exception& e_ ) {
		cerr << "While defining program options: " << e_.what() << "\n";
		return 1;
	}
	a->setCommand(command);
	a->printOut();
	delete a;
	return 0;
}
